SuspendObject subclass: #SuspendBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Suspend-Ex'!


!SuspendBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger
	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!SuspendBoundedBuffer methodsFor: 'script'!

get
	"Wait until buffer is not empty."

	self waitUntil: [contents isEmpty not].
	^contents removeFirst!

put: item
	"Wait until buffer is not full."

	self waitUntil: [contents isFull not].
	contents addLast: item! !

!SuspendBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SuspendBoundedBuffer class
	instanceVariableNames: ''!


!SuspendBoundedBuffer class methodsFor: 'instance creation'!

new: size
	^self new initialize: size! !

!SuspendBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

SuspendBoundedBuffer subclass: #SuspendGgetBoundedBuffer
	instanceVariableNames: 'afterPut '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Suspend-Ex'!


!SuspendGgetBoundedBuffer methodsFor: 'script'!

gget
	"We cannot reuse condition or call of method get."

	self waitUntil: [(contents isEmpty not) & (afterPut not)].
	^contents removeFirst! !

!SuspendGgetBoundedBuffer methodsFor: 'events'!

eventComplete: aMessage
	afterPut := aMessage selector = #put:.
	Transcript show: self printString , ' complete (' , aMessage compactPrintString , ')'; cr!

eventReceive: aMessage
	Transcript show: self printString , ' receive (' , aMessage compactPrintString , ')'; cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SuspendGgetBoundedBuffer class
	instanceVariableNames: ''!


!SuspendGgetBoundedBuffer class methodsFor: 'example'!

exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGget: numberItems! !

SuspendBoundedBuffer subclass: #SuspendGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Suspend-Ex'!


!SuspendGet2BoundedBuffer methodsFor: 'script'!

get2
	"Wait until buffer contains at least two elements."

	self waitUntil: [contents size >= 2].
	^Array
		with: self get
		with: self get! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SuspendGet2BoundedBuffer class
	instanceVariableNames: ''!


!SuspendGet2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet2: numberItems! !
